/***************************************************
 *
 * "THE DON'T-CARE LICENCE"
 * I don't care, do whatever you want with this file.
 *
 * www.elektronika.ba
 *
****************************************************/

// README
// Limitations: PORTB.4 can only be used as INPUT.

#define F_CPU 8000000

// If defined, the booster doesn't read the feedback voltage
//#define	DEBUG_BOOST_VAL		243		// the OCR0A value for manual setting

// Booster HW
#define	BOOST_PIN			5		// if changed, adjust TCCR0A accordingly (in wt.c)
#define	BOOST_DDR			DDRD
#define	BOOST_PORT			PORTD


// Booster Feedback A/D PIN
#define	FEEDBACK_PIN		0
#define	FEEDBACK_DDR		DDRC
#define	FEEDBACK_DIDR		DIDR0
#define	FEEDBACK_DIDR_VAL	0x01 // ADC0 digitals disabled
#define	FEEDBACK_ADMUX_VAL	0b00000000 // AREF (internal Vref turned off), ADC0


// Power Sense HW
#define	PWR_SENS_PIN		1
#define	PWR_SENS_DDR		DDRC
#define	PWR_SENS_PINREG		PINC
#define	PWR_SENS_PORT		PORTC
#define	PWR_SENS_PCINTBIT	PCINT9
#define	PWR_SENS_PCMSKREG	PCMSK1
#define	PWR_SENS_PCICRBIT	PCIE1


// Buttons and switch
#define	BTNA_PIN			5
#define	BTNA_DDR			DDRC
#define	BTNA_PINREG			PINC
#define	BTNA_PORT			PORTC
#define	BTNA_PCINTBIT		PCINT13
#define	BTNA_PCMSKREG		PCMSK1
#define	BTNA_PCICRBIT		PCIE1

#define	BTNB_PIN			2
#define	BTNB_DDR			DDRC
#define	BTNB_PINREG			PINC
#define	BTNB_PORT			PORTC
#define	BTNB_PCINTBIT		PCINT10
#define	BTNB_PCMSKREG		PCMSK1
#define	BTNB_PCICRBIT		PCIE1

#define	SWALARM_PIN			4
#define	SWALARM_DDR			DDRB
#define	SWALARM_PINREG		PINB
#define	SWALARM_PORT		PORTB
#define	SWALARM_PCINTBIT	PCINT4
#define	SWALARM_PCMSKREG	PCMSK0
#define	SWALARM_PCICRBIT	PCIE0


// SN74141 BCD Decoder
#define	SN74141_PINS		0b00011110 // where SN's A,B,C,D pins are located on AVR's port. the order is important in "sn74141_map" below
#define	SN74141_DDR			DDRD
#define	SN74141_PORT		PORTD

// Nixie lower dot
#define	NIXIE_LDOT_PIN		3
#define	NIXIE_LDOT_DDR		DDRC
#define	NIXIE_LDOT_PORT		PORTC

// Nixie upper dot
#define	NIXIE_UDOT_PIN		4
#define	NIXIE_UDOT_DDR		DDRC
#define	NIXIE_UDOT_PORT		PORTC


// Sound speaker
#define	SOUND_PIN			1
#define	SOUND_DDR			DDRB
#define	SOUND_PORT			PORTB


// TLC59401 HW
#define GSCLK_DDR			DDRB
#define GSCLK_PORT			PORTB
#define GSCLK_PIN			0

#define XLAT_DDR			DDRD
#define XLAT_PORT			PORTD
#define XLAT_PIN			6

#define BLANK_DDR			DDRB
#define BLANK_PORT			PORTB
#define BLANK_PIN			2

#define SIN_DDR				DDRB
#define SIN_PORT			PORTB
#define SIN_PIN				3

#define SCLK_DDR			DDRB
#define SCLK_PORT			PORTB
#define SCLK_PIN			5

#define MODE_DDR			DDRD
#define MODE_PORT			PORTD
#define MODE_PIN			7


// Code macros
#define set0(port, pin)			( (port) &= (uint8_t)~_BV(pin) )
#define set1(port, pin)			( (port) |= (uint8_t)_BV(pin) )


// Boolean macros
#define	BSYS					0										// bank index[0] name.. used for system work
#define	BAPP1					1										// bank index[1] name.. used for application work
#define	BAPP2					2										// bank index[2] name.. used for application work
#define	BUSR1					3										// bank index[3] name.. custom, for user settings
#define BOOL_BANK_SIZE			4										// two banks (bytes) .. 16 bits
#define bv(bit,bank) 			( bools[bank] & (uint8_t)_BV(bit) ) 	// boolean values GET/READ macro: if(bv(3,BSYS))...
#define	bs(bit,bank) 			( bools[bank] |= (uint8_t)_BV(bit) ) 	// boolean values SET macro: bs(5,BSYS)
#define	bc(bit,bank) 			( bools[bank] &= (uint8_t)~_BV(bit) ) 	// boolean values CLEAR macro: bc(4,BAPP)
#define	bt(bit,bank) 			( bools[bank] ^= (uint8_t)_BV(bit) ) 	// boolean values TOGGLE macro: bt(6,BAPP)


// Display setup
#define DISPLAY_REFRESH_05MS	6		// (4 for IN-8; 6 for IN-16!) 1/2 ms display refresh interval, this also clocks-in new data to TLC chip from the whole application. this has to be 1/2 of non-flickering frequency (which is chosen for a specific tube) WHEN cross fading is used!!!
#define	DISPLAY_SIZE			4		// how many nixie tubes are available?

// Display misc. - MIN value is nixie tube dependent
#define	DISP_MAX_BRIGHTNESS		4095	// maximum allowed brightness for display brightness setting.. this is (100%) how much it goes (TLC59401 specs.)
/*
info:
in-8 @ refresh rate=4, last_off=3840 (3860?)
in-16 @ refresh rate=6, last_off=3843
*/
#define	DISP_LAST_OFF_BRTNS		3840	// IN-8:3840, IN-16:3843 - Depends on Nixie tube type & (probably) on: DISPLAY_REFRESH_05MS, so try choosing a valid value.. this should actually be a 0, but a strange thing happens with multiplexed tubes and there are values>0 when nixie tube is OFF. this is the last value in row that blanks a nixie tube! (the first one under 4095)
#define	DISP_MIN_BRIGHTNESS		(DISP_LAST_OFF_BRTNS+7)		// minimum allowed brightness for display brightness setting.. +7 so user doesn't turn the display off completelly!

//#define	CROSS_FADING_CODE		// when defined, the cross-fading code is compiled and enabled
#define	DEFAULT_CROSS_FAD_SPD	10		// default speed of cross fader (normally this is between 1 and 10, but not limited to any value)

// Display driver setup, PCB dependent
#define	TLC_LAST_NIXIE_CH		9		// last TLC channel where first nixie tube ([H]H:MM)(opto-coupler) is (range: 0..15-DISPLAY_SIZE) others must follow in backwards fashion MM:HH (this is because of the PCB design)

// Custom boolean bit/flags, 0..7 in bank 1 - SYSTEM SPECIFIC
#define TLC_HasDCUpdate			0		// does TLC chip need XLAT?
#define TLC_HasGSUpdate			1		// does TLC chip need XLAT?
#define TLC_NeedsExtraCLK		2		// does TLC chip need extra CLK impulse after the GS latch?
//#define	TLC_GSChanged			3		// for the _ClockInGS() function, so it doesn't clock the TLC chip if there's nothing new to send it
#define	SN74141_Update			4		// flag for updating the SN74141
#define	SleepMode				5		// is device in sleep mode?
#define	AutoOffed				6		// is device in auto-off mode?
#define	BoosterOnline			7		// is booster working?

// Another boolean bit/flags, 0..7 in bank 2 - APPLICATION SPECIFIC - 1
#define	ToggleColons			0		// can we toggle Nixie Colons at 1/2 Hz rate
#define	ToggleLEDs				1		// can we toggle bottom LEDs at 1/2 Hz rate
#define	RTC_AlarmOn				2		// alarm on/off
#define	RTC_Alarming			3		// alarming as we speak
#define	AntiPoison				4		// time to anti-poison the nixies?
#define RTC_IsSet				5		// is time/date set?
#define	BlankTime				6		// should display_time() display the current time or not?
#define RTC_Snoozing			7		// is snooze activated?

// Another boolean bit/flags, 0..7 in bank 3 - APPLICATION SPECIFIC - 2
#define	CrossFaderEn			0		// can cross-fader be enabled or not - at this moment in the application

// Another boolean bit/flags, 0..7 in bank 4 - USER PREFERENCES
#define	LEDtoggling				0		// does user want the bottom LEDs to toggle in 1/2 Hz rate in time mode?
#define	TEMPblender				1		// does user want the bottom LEDs to produce a temperature color when in temperature mode?
#define	LEDs4date				2		// does user want the bottom LEDs to turn on during date display?
#define	DateFading				3		// does user want the faded DD.MM display instead of normal one?
#define	AutoOffMode				4		// does user want the auto-off mode enabled?
#define	CrossFading				5		// does user want the cross-fading effect?


// Snooze stuff
#define	DEFAULT_SNOOZE_INTERVAL	5		// 5 mins
#define	RTC_AUTO_SNOOZE			60		// after alarming for this many seconds, snooze automatically so we don't annoy forever


// Default color of bottom LEDs when toggling them in 1/2 Hz rate
#define TimeLEDColorR			0
#define TimeLEDColorG			190
#define TimeLEDColorB			0

// Default color of bottom LEDs when showing date
#define DateLEDColorR			20
#define DateLEDColorG			0
#define DateLEDColorB			0


// How many things can be displayed on display in "idle" mode and their timeouts
#define DISP_THINGS				3		// 0-time, 1-date, 2-temp
#define	DEFAULT_DISP_0			15		// seconds for time default
#define	DEFAULT_DISP_1			0		// seconds for date default (it has it's own internal delays so 1 sec is enough)
#define	DEFAULT_DISP_2			2		// seconds for temperature default


// RTC name constants for indexing the RTC[0..5] and RTC_ALARM[0..1] array
#define	TIME_H					0		// also used for RTC_ALARM
#define	TIME_M					1		// also used for RTC_ALARM
#define	TIME_S					2
#define	DATE_D					3
#define	DATE_M					4
#define	DATE_Y					5

// Button related timers
#define	BTN_INACTIVITY_TMR		5000	// miliseconds to pronounce inactivity of button input
#define	BTN_HOLD_TMR			850		// miliseconds to pronounce button as held rather than pressed
#define	BTN_REPEAT_RATE			140		// miliseconds is a speed of "fast incrementing a value" during setup... aka the "repetition rate"

// Delay used in display_date() in milliseconds
#define SHOW_DATE_DDMM_DLY		2000	// 2000ms for DDMM part (DD. is /2, .MM is /2)
#define SHOW_DATE_YYYY_DLY		2000	// 2000ms for YYYY part

// Temperature color blender setup
#define ColorBlender_Kb			20		// different colors in selected range
// Coldest situation (cool blue color):
#define	Tmin					10		// (>0 !) minimum temperature for the Coldest situation
#define Rmin					0		// color
#define Gmin					50		// color
#define Bmin					255		// color
// Hottest situation (hot red color):
#define	Tmax					35		// (>0 !) maximum temperature for the Hottest situation, yeah baby
#define Rmax					255		// color
#define Gmax					0		// color
#define Bmax					0		// color

// EEPROM addresses for settings and things
#define	EEPROM_START			0		// start of our data in eeprom is here
/*
#define	EE_EEWRITTEN			0		// byte
#define	EE_RTC					1		// 6 bytes in block
#define	EE_RTC_ALARM			7		// 2 bytes in block
#define	EE_RTC_AOFF				9		// 2 bytes in block
#define	EE_RTC_AON				11		// 2 bytes in block
#define	EE_RGB_TIME_COLOR		13		// 3 bytes in block
#define	EE_RGB_DATE_COLOR		16		// 3 bytes in block
#define EE_MAX_BRIGHTNESS		19		// word
#define	EE_DATE_FADING_BRIGHTN	21		// word
#define EE_SNOOZE_INTERVAL 		23		// byte
#define	EE_DISP_DURATION		24		// DISP_THINGS bytes in block
#define	EE_BOOLS				(24+DISP_THINGS)// BOOL_BANK_SIZE bytes in block
//#define	EE_SOMETHING		(24+DISP_THINGS+BOOL_BANK_SIZE)// ...
*/

// RGB LED mapping to TLC59401 channels
// RED,GREEN,BLUE parts of LEDs
/*
// IN-8 Nixies @ ?,?,?,?
const char rgb_tlc_map[] PROGMEM = { // char instead of uint8_t because of gcc warning
	// todo this table
	12,15,02,05, // R: led1, led2, led3, led4
	11,14,01,04, // G: led1, led2, led3, led4
	10,13,00,03, // B: led1, led2, led3, led4
};
*/
// IN-16 Nixies @ 9,8,7,6
const char rgb_tlc_map[] PROGMEM = { // char instead of uint8_t because of gcc warning
	11,14,01,04, // R: led1, led2, led3, led4
	10,13,00,03, // G: led1, led2, led3, led4
	12,15,02,05, // B: led1, led2, led3, led4
};
PGM_P const rgb_tlc_map_p PROGMEM = rgb_tlc_map;

// SN74141 BCD decoder mapping to "SN74141_DDR" AVR port
/*
// IN-8 final pcb
const char sn74141_map[] PROGMEM = { // char instead of uint8_t because of gcc warning
	0b00010000, // 0
	0b00001000, // 1
	0b00001010, // 2
	0b00000110, // 3
	0b00000010, // 4
	0b00010010, // 5
	0b00011000, // 6
	0b00011010, // 7
	0b00000100, // 8
	0b00000000, // 9
	0b00001100, // blank .. any value >=10
};
*/
/*
// IN-16 final pcb
const char sn74141_map[] PROGMEM = { // char instead of uint8_t because of gcc warning
	0b00000000, // 0
	0b00010000, // 1
	0b00001000, // 2
	0b00001010, // 3
	0b00000110, // 4
	0b00000010, // 5
	0b00010010, // 6
	0b00011000, // 7
	0b00011010, // 8
	0b00000100, // 9
	0b00001100, // blank .. any value >=10
};
*/
// Custom PCB, not official
const char sn74141_map[] PROGMEM = { // char instead of uint8_t because of gcc warning
	0b00000000, // 0
	0b00010000, // 1
	0b00000100, // 2
	0b00010100, // 3
	0b00000010, // 4
	0b00010010, // 5
	0b00000110, // 6
	0b00010110, // 7
	0b00001000, // 8
	0b00011000, // 9
	0b00001100, // blank .. any value >=10
};

PGM_P const sn74141_map_p PROGMEM = sn74141_map;
#define nixie_blank	10 // item index in sn74141_map that has invalid value

// Alarm melody, format: tone(0-255), duration(0-65535ms)
// Note: putting 0 in duration will reset the melody to beginning.. so don't do it!
const uint16_t melody[] PROGMEM = {
	10, 70,
	0, 50,
	20, 40,
	0, 50,

	10, 70,
	0, 50,
	20, 40,
	0, 50,

	10, 70,
	0, 50,
	20, 40,
	0, 50,

	10, 70,
	0, 50,
	20, 40,

	0, 1000,
};
#define	MELODY_LENGTH	(16*2)			// how many tones in melody array *2? (number of items in array)
